@@ -110,6 +110,21 @@ def create_parser(protocol_name):
     return parser_cls()
 
 
+def _text_content(func):
+    # This decorator hides the difference between
+    # an XML node with text or a plain string.  It's used
+    # to ensure that scalar processing operates only on text
+    # strings, which allows the same scalar handlers to be used
+    # for XML nodes from the body and HTTP headers.
+    def _get_text_content(self, shape, node_or_string):
+        if hasattr(node_or_string, 'text'):
+            text = node_or_string.text
+        else:
+            text = node_or_string
+        return func(self, shape, text)
+    return _get_text_content
+
+
 class ResponseParserError(Exception):
     pass
 
@@ -191,6 +206,8 @@ class BaseXMLResponseParser(ResponseParser):
         value_shape = shape.value
         key_location_name = key_shape.serialization.get('name') or 'key'
         value_location_name = value_shape.serialization.get('name') or 'value'
+        if shape.serialization.get('flattened') and not isinstance(node, list):
+            node = [node]
         for keyval_node in node:
             for single_pair in keyval_node:
                 # Within each <entry> there's a <key> and a <value>
@@ -284,25 +301,30 @@ class BaseXMLResponseParser(ResponseParser):
                 parsed[key] = value.text
         return parsed
 
-    def _handle_boolean(self, shape, node):
-        if node.text == 'true':
+    @_text_content
+    def _handle_boolean(self, shape, text):
+        if text == 'true':
             return True
         else:
             return False
 
-    def _handle_float(self, shape, node):
-        return float(node.text)
+    @_text_content
+    def _handle_float(self, shape, text):
+        return float(text)
 
-    def _handle_timestamp(self, shape, node):
-        if hasattr(node, 'text'):
-            value = node.text
-        else:
-            value = node
-        return self._timestamp_parser(value)
+    @_text_content
+    def _handle_timestamp(self, shape, text):
+        return self._timestamp_parser(text)
 
-    def _handle_integer(self, shape, node):
-        return int(node.text)
+    @_text_content
+    def _handle_integer(self, shape, text):
+        return int(text)
 
+    @_text_content
+    def _handle_string(self, shape, text):
+        return text
+
+    _handle_character = _handle_string
     _handle_double = _handle_float
     _handle_long = _handle_integer
 
@@ -430,9 +452,11 @@ class JSONParser(BaseJSONParser):
         # The json.loads() gives us the primitive JSON types,
         # but we need to traverse the parsed JSON data to convert
         # to richer types (blobs, timestamps, etc.
-        body = response['body'].decode(self.DEFAULT_ENCODING)
-        original_parsed = json.loads(body)
-        parsed = self._parse_shape(shape, original_parsed)
+        parsed = {}
+        if shape is not None:
+            body = response['body'].decode(self.DEFAULT_ENCODING)
+            original_parsed = json.loads(body)
+            parsed = self._parse_shape(shape, original_parsed)
         self._inject_response_metadata(parsed, response['headers'])
         return parsed
 
@@ -638,33 +662,9 @@ class RestXMLParser(BaseRestParser, BaseXMLResponseParser):
             parsed['ResponseMetadata'] = {'RequestId': parsed.pop('RequestId')}
         return parsed
 
-    def _handle_float(self, shape, node):
-        if hasattr(node, 'text'):
-            return float(node.text)
-        else:
-            return float(node)
-
-    def _handle_integer(self, shape, node):
-        if hasattr(node, 'text'):
-            return int(node.text)
-        else:
-            return int(node)
-
     def _handle_blob(self, shape, node):
         return base64.b64decode(node.text)
 
-    def _default_handle(self, shape, node):
-        if hasattr(node, 'text'):
-            # If it's an xml node, return the text of the
-            # xml node.
-            return node.text
-        else:
-            # Otherwise, return the passed in string/int/scalar type.
-            return node
-
-    _handle_double = _handle_float
-    _handle_long = _handle_integer
-
 
 PROTOCOL_PARSERS = {
     'ec2': EC2QueryParser,
